///////////////////////////////////////////////////////////////////////////////////////////////////
// LCOM.h : LCOM.DLL Library Header for User Interface
//
//                              Copyright(c) 2015
//                  LSMecapion Co, Ltd. - SEOUL, REPUBLIC OF KOREA
//
// This program is the property of the LSMecapion Co, Ltd. and it shall not be reproduced, 
// distributed or used without permission of an authorized Company official. 
// This is an unpublished work subject to Trade Secret and Copyright protection.
//
///////////////////////////////////////////////////////////////////////////////////////////////////]

#ifndef __LCOMDLL_HEADER__
#define __LCOMDLL_HEADER__

#ifdef __cplusplus
extern "C" 
{
#endif

#define DLL_MAKER

#ifdef  DLL_MAKER
#define DLLFUNC extern __declspec(dllexport) 
#else
#define DLLFUNC __declspec(dllimport) 
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////
#define LCOM_DEVICE_CON_MAX		32		// ̺  ִ 
#define LCOM_DEVICE_ID_MAX		0xFF    // ̺ ID ִ  (0~0xFF)
#define LCOM_DEVICE_ID_CNT_MAX	(LCOM_DEVICE_ID_MAX+1)
#define LCOM_DEVICE_ID_NULL		0

#define EMPTY				0		// Drive Type is not assigned
#define SDRV				1		// Drive Type for Serial Communication
#define EDRV				2		// Drive Type for Ethernet Communication

#ifndef MAX_RX_BUFFER
#define	MAX_RX_BUFFER		65
#endif
#ifndef MAX_TX_BUFFER
#define	MAX_TX_BUFFER		65
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////
// LCOM Library Error Code
enum
{
	LCOM_OK = 0,
	LCOM_INIT_ERR,
	LCOM_LIBRARY_ERR,
	LCOM_PARAMETER_INVALID_ERR,

	LCOM_MEI_REPLY_UNKNOWN_CODE_ERR,
	LCOM_MEI_REPLY_ILLEGAL_FUNCTION_ERR,
	LCOM_MEI_REPLY_ILLEGAL_DATA_ADDRESS_ERR,
	LCOM_MEI_REPLY_ILLEGAL_DATA_VALUE_ERR,
	LCOM_MEI_REPLY_SERVICE_DEVICE_FAILURE_ERR,
	LCOM_MEI_REPLY_ACK_ERR,
	LCOM_MEI_REPLY_SERVICE_DEVICE_BUSY_ERR,
	LCOM_MEI_REPLY_MEMORY_PARITY_ERR,

	LCOM_MEI_SERIAL_OPEN_ERR,
	LCOM_MEI_SERIAL_CLOSE_ERR,
	LCOM_MEI_SERIAL_NORESPONSE_ERR,
	LCOM_MEI_SEEIAL_WRITE_ERR,
	LCOM_MEI_SERIAL_WRITE_ERR,
	LCOM_MEI_SERIAL_CRC_ERR,
	LCOM_MEI_SERIAL_REPLYCMD_ERR,
	LCOM_MEI_SERIAL_REPLYSTS_ERR,
	LCOM_MEI_SERIAL_TIMEOUT_ERR,
	LCOM_MEI_SERIAL_RXPACKET_MISSING_ERR,
	LCOM_MEI_SERIAL_REQUEST_BYTESIZE_TOOBIG_ERR,
	LCOM_MEI_SERIAL_HANDLE_ERR,

	LCOM_MEI_ETHERNET_OPEN_ERR,
	LCOM_MEI_ETHERNET_CLOSE_ERR,
	LCOM_MEI_ETHERNET_NORESPONSE_ERR,
	LCOM_MEI_ETHERNET_WRITE_ERR,
	LCOM_MEI_ETHERNET_CRC_ERR,
	LCOM_MEI_ETHERNET_REPLYCMD_ERR,
	LCOM_MEI_ETHERNET_REPLYSTS_ERR,
	LCOM_MEI_ETHERNET_TIMEOUT_ERR,
	LCOM_MEI_ETHERNET_RXPACKET_MISSING_ERR,
	LCOM_MEI_ETHERNET_REQUEST_BYTESIZE_TOOBIG_ERR,
	LCOM_MEI_ETHERNET_SOCKET_NULL_ERR,
	LCOM_MEI_ETHERNET_SOCKET_ALREADY_USED_ERR,
	LCOM_MEI_ETHERNET_DRIVEID_NOTACTIVE_ERR,
	LCOM_MEI_ETHERNET_DRIVEID_MISMATCH_ERR,
	LCOM_CONNECTION_EMPTY_ERR,
	LCOM_CONNECTION_SLALVE_ONLINE_ERR,
	LCOM_NO_EMPTY_CONNECTION_ERR,
	LCOM_DRIVEID_RANGE_ERR,
	LCOM_PARAMETER_STRSIZE_ERR,

	LCOM_FAIL=-1,
};
#pragma pack(1)
typedef struct IndexMotionProfileSt
{
	int				IndexNo;
	unsigned short	Type;
	long			Dist;
	long			Vel;
	long			Acc;
	long			Dec;
	long			RegDist;
	long			RegVel;
	unsigned short	Repeat;
	unsigned short	Dwell;
	unsigned short	Next;
	unsigned short	Action;
}INDEXMOTION_PROFILE, *PINDEXMOTION_PROFILE;
#pragma pack(4)

///////////////////////////////////////////////////////////////////////////////////////////////////
//DLLFUNC int __stdcall LCOM_DriveOpen(int DriveId, int DrvType, char *pComPortOrIpAddress, int dwBaudOrPortNum);
DLLFUNC int __stdcall LCOM_SDRVOpen(int DriveId, char *pComPort, int dwBaud);
DLLFUNC int __stdcall LCOM_EDRVOpen(int DriveId, char *IpAddress, int PortNum);
DLLFUNC int __stdcall LCOM_DriveClose(int DriveId);

DLLFUNC int __stdcall LCOM_GetEoESetupFile(char* pFileName);
DLLFUNC int __stdcall LCOM_SetEoESetupFile(char* pFileName);
DLLFUNC int __stdcall LCOM_GetOnlineMode(int* Mode);
DLLFUNC int __stdcall LCOM_IsSlaveOnline(int DriveId, int* State);
DLLFUNC int __stdcall LCOM_GetSlaveCount(int* Count);

// utility
DLLFUNC int	__stdcall LCOM_GetErrorCode(char *cErrorString);
DLLFUNC int	__stdcall LCOM_GetErrorString( char *pErr);
DLLFUNC int __stdcall LCOM_BinToStr( char* pData, unsigned char* pTarget, int nLen );
DLLFUNC int __stdcall LCOM_StrToBin( char *pSrc, unsigned char* pTarget );
DLLFUNC int __stdcall LCOM_GetTxPackets(BYTE DriveId, unsigned char* pBuffer, unsigned short* pTxSize );
DLLFUNC int __stdcall LCOM_GetRxPackets(BYTE DriveId, unsigned char* pBuffer, unsigned short* pRxSize );

// parameter read/write
DLLFUNC int __stdcall LCOM_ReadByte(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, unsigned char* pResult);
DLLFUNC int __stdcall LCOM_ReadByteS(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, char* pResult);
DLLFUNC int __stdcall LCOM_ReadWord(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, unsigned short* pResult);
DLLFUNC int __stdcall LCOM_ReadWordS(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, short* pResult);
DLLFUNC int __stdcall LCOM_ReadDWord(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, unsigned long* pResult);
DLLFUNC int __stdcall LCOM_ReadDWordS(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, signed long* pResult);
DLLFUNC int __stdcall LCOM_ReadString(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, char* pResult);
DLLFUNC int __stdcall LCOM_WriteByte(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, unsigned char Data);
DLLFUNC int __stdcall LCOM_WriteByteS(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, char Data);
DLLFUNC int __stdcall LCOM_WriteWord(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, unsigned short Data);
DLLFUNC int __stdcall LCOM_WriteWordS(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, short Data);
DLLFUNC int __stdcall LCOM_WriteDWord(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, unsigned long Data);
DLLFUNC int __stdcall LCOM_WriteDWordS(unsigned char DestID, unsigned short Addr, unsigned char SubIndex, signed long Data);
DLLFUNC int __stdcall LCOM_ProcedureCmd(unsigned char DestID, unsigned short Cmd, unsigned short Arg);
	
// common
DLLFUNC int __stdcall LCOM_GetLcomDllVersion(int* pVersion);
DLLFUNC int __stdcall LCOM_GetTimeout(int* pRxTimeout, int*pConnectionTimeout);
DLLFUNC int __stdcall LCOM_SetTimeout(int RxTimeout, int ConnectionTimeout);

//control parameter
DLLFUNC int __stdcall LCOM_SetControlMode(unsigned char DestID, unsigned short Mode);
DLLFUNC int __stdcall LCOM_SetOperationMode(unsigned char DestID, char Mode); //operation mode bae.
DLLFUNC int __stdcall LCOM_SetElecGearRatio(unsigned char DestID, unsigned long MotorRev, unsigned long ShaftRev);
DLLFUNC int __stdcall LCOM_SetCurGain(unsigned char DestID, unsigned short Gain);
DLLFUNC int __stdcall LCOM_SetInertiaRatio(unsigned char DestID, unsigned short Ratio);
DLLFUNC int __stdcall LCOM_SetVelGainKp(unsigned char DestID, unsigned short Gain);
DLLFUNC int __stdcall LCOM_SetVelGainKiT(unsigned char DestID, unsigned short Gain);
DLLFUNC int __stdcall LCOM_SetPosGainKp(unsigned char DestID, unsigned short Gain);
DLLFUNC int __stdcall LCOM_SetPosGainKvff(unsigned char DestID, unsigned short Gain);
DLLFUNC int __stdcall LCOM_SetPosGainKvff_flt(unsigned char DestID, unsigned short Gain);
DLLFUNC int __stdcall LCOM_GetCurGain(unsigned char DestID, unsigned short* Gain);
DLLFUNC int __stdcall LCOM_GetInertiaRatio(unsigned char DestID, unsigned short* Ratio);
DLLFUNC int __stdcall LCOM_GetVelGainKp(unsigned char DestID, unsigned short* Gain);
DLLFUNC int __stdcall LCOM_GetVelGainKiT(unsigned char DestID, unsigned short* Gain);
DLLFUNC int __stdcall LCOM_GetPosGainKp(unsigned char DestID, unsigned short* Gain);
DLLFUNC int __stdcall LCOM_GetPosGainKvff(unsigned char DestID, unsigned short* Gain);
DLLFUNC int __stdcall LCOM_GetPosGainKvff_flt(unsigned char DestID, unsigned short* Gain);

//monitor
DLLFUNC int __stdcall LCOM_FeedbackSpeed(unsigned char DestID, short* Vel);
DLLFUNC int __stdcall LCOM_ActualPosition(unsigned char DestID, long* Pos);
DLLFUNC int __stdcall LCOM_ActualVelocity(unsigned char DestID, long* Vel);
DLLFUNC int __stdcall LCOM_ActualTorque(unsigned char DestID, short* Torque);
DLLFUNC int __stdcall LCOM_SingleTurnData(unsigned char DestID, unsigned long* Data);
DLLFUNC int __stdcall LCOM_MultiTurnData(unsigned char DestID, unsigned short* Data);
DLLFUNC int __stdcall LCOM_WarningCode(unsigned char DestID, unsigned short* Code);
DLLFUNC int __stdcall LCOM_DCLinkVoltage(unsigned char DestID, unsigned short* Volt);
DLLFUNC int __stdcall LCOM_AccumOverload(unsigned char DestID, short* Overload);
DLLFUNC int __stdcall LCOM_DriveTemp(unsigned char DestID, short* Temp);
DLLFUNC int __stdcall LCOM_ResetAlarm(unsigned char DestID);
DLLFUNC int __stdcall LCOM_IsAlarm(unsigned char DestID, int* State, unsigned short* Code);
DLLFUNC int __stdcall LCOM_GetAlarmHistory(unsigned char DestID, unsigned char SubIndex, char* Buf);
DLLFUNC int __stdcall LCOM_ClearAlarmHistory(unsigned char DestID);
DLLFUNC int __stdcall LCOM_DriveSoftReset(unsigned char DestID);
DLLFUNC int __stdcall LCOM_StoreParameter(unsigned char DestID);
//I/O handling
DLLFUNC int __stdcall LCOM_SetDigitalInputSignals(unsigned char DestID, unsigned char SignalNo, unsigned short Signal, unsigned short Level);
DLLFUNC int __stdcall LCOM_GetDigitalInputSignals(unsigned char DestID, unsigned char SignalNo, unsigned short* Signal, unsigned short* Level);
DLLFUNC int __stdcall LCOM_SetDigitalOutputSignals(unsigned char DestID, unsigned char SignalNo, unsigned short Signal, unsigned short Level);
DLLFUNC int __stdcall LCOM_GetDigitalOutputSignals(unsigned char DestID, unsigned char SignalNo, unsigned short* Signal, unsigned short* Level);
DLLFUNC int __stdcall LCOM_GetDigitalInputs(unsigned char DestID, unsigned long* Val);
DLLFUNC int __stdcall LCOM_IsNegativeLimit(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_IsPositiveLimit(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_IsHomeSensor(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_GetDigitalOutputs(unsigned char DestID, unsigned long* Val);
DLLFUNC int __stdcall LCOM_SetDigitalOutputsWithMask(unsigned char DestID, unsigned long Val, unsigned long Mask);
DLLFUNC int __stdcall LCOM_GetDriveControlInput1(unsigned char DestID, int BitIndex, int* State);
DLLFUNC int __stdcall LCOM_GetDriveControlInput2(unsigned char DestID, int BitIndex, int* State);
DLLFUNC int __stdcall LCOM_SetDriveControlInput1(unsigned char DestID, int BitIndex, int State);
DLLFUNC int __stdcall LCOM_SetDriveControlInput2(unsigned char DestID, int BitIndex, int State);
DLLFUNC int __stdcall LCOM_GetDriveStateOutput(unsigned char DestID, int BitIndex, int* State);
DLLFUNC int __stdcall LCOM_TouchProbeFunction(unsigned char DestID, unsigned short Val);
DLLFUNC int __stdcall LCOM_TouchProbeStatus(unsigned char DestID, unsigned short* Status);
DLLFUNC int __stdcall LCOM_TouchProbe1PositiveEdgeValue(unsigned char DestID, long* Val);
DLLFUNC int __stdcall LCOM_TouchProbe1NegativeEdgeValue(unsigned char DestID, long* Val);
DLLFUNC int __stdcall LCOM_TouchProbe2PositiveEdgeValue(unsigned char DestID, long* Val);
DLLFUNC int __stdcall LCOM_TouchProbe2NegativeEdgeValue(unsigned char DestID, long* Val);
//homing
DLLFUNC int __stdcall LCOM_Homing(unsigned char DestID,char Method, long Speed, long FineSpeed, unsigned long Acc, short Offset);
DLLFUNC int __stdcall LCOM_IsHome(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_HomeTorqueLimit(unsigned char DestID, unsigned short Torque, unsigned short Time);
//amp
DLLFUNC int __stdcall LCOM_Enable(unsigned char DestID);
DLLFUNC int __stdcall LCOM_Disable(unsigned char DestID);
DLLFUNC int __stdcall LCOM_IsEnabled(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_EmergencyStop(unsigned char DestID);
DLLFUNC int __stdcall LCOM_Stop(unsigned char DestID);
DLLFUNC int __stdcall LCOM_Start(unsigned char DestID);
//ptp motion
DLLFUNC int __stdcall LCOM_SetCurrentPosition(unsigned char DestID, long HomeOffset);
DLLFUNC int __stdcall LCOM_MoveAbsolute(unsigned char DestID, signed long Pos, unsigned long Vel, unsigned long Acc, unsigned long Dec);
DLLFUNC int __stdcall LCOM_MoveAbs(unsigned char DestID, signed long Pos);
DLLFUNC int __stdcall LCOM_MoveRelative(unsigned char DestID, signed long Pos, unsigned long Vel, unsigned long Acc, unsigned long Dec);
DLLFUNC int __stdcall LCOM_MoveRel(unsigned char DestID, signed long Pos);
//index motion
DLLFUNC int __stdcall LCOM_MoveIndex(unsigned char DestID, int IndexNo);
DLLFUNC int __stdcall LCOM_ReadIndexParameter(unsigned char DestID, int IndexNo, INDEXMOTION_PROFILE *pInfo);
DLLFUNC int __stdcall LCOM_WriteIndexParameter(unsigned char DestID, int IndexNo, INDEXMOTION_PROFILE *pInfo);
DLLFUNC int __stdcall LCOM_IsEndIndex(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_SetInPosition(unsigned char DestID, unsigned long PosWindow, unsigned short TimeWindow);
DLLFUNC int __stdcall LCOM_IsInPosition(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_WaitMoveDone(unsigned char DestID, int WaitTime, int* State);
DLLFUNC int __stdcall LCOM_TargetVelocity(unsigned char DestID, long Vel);
DLLFUNC int __stdcall LCOM_ProfileVelocity(unsigned char DestID, unsigned long Vel);
DLLFUNC int __stdcall LCOM_ProfileAccel(unsigned char DestID, unsigned long Acc);
DLLFUNC int __stdcall LCOM_ProfileDecel(unsigned char DestID, unsigned long Dec);
DLLFUNC int __stdcall LCOM_StopDecel(unsigned char DestID, unsigned long Dec);
DLLFUNC int __stdcall LCOM_SetInVelocity(unsigned char DestID, unsigned long VelWindow, unsigned short TimeWindow);
DLLFUNC int __stdcall LCOM_IsInVelocity(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_TargetTorque(unsigned char DestID, long Torque);
DLLFUNC int __stdcall LCOM_TorqueVelLimit(unsigned char DestID, unsigned long TorqueVelLimit);
DLLFUNC int __stdcall LCOM_TorqueSlope(unsigned char DestID, unsigned long TorqueSlope);
DLLFUNC int __stdcall LCOM_IsTorqueLimit(unsigned char DestID, int* State);
DLLFUNC int __stdcall LCOM_SelVelocityLimit(unsigned char DestID, unsigned short Select);
DLLFUNC int __stdcall LCOM_SetVelocityLimit(unsigned char DestID, unsigned short VelocityLimit);

#ifdef __cplusplus
}
#endif

#endif
